==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/server/inc/xml.php <==
<?php

/**
  @class XML
  @brief Class for editing and querying XML documents.
  @author Josef Hala
  @author Martin Zlomek
  @date 21-02-2006
 */
class XML extends DOMDocument {
	/** Name of the document file. */
	private $file;

	/** XPath object of the document. */
	private $xpath;

	/**
	  @brief Create a new XML document object.
	  @param $file Optional name of the document file.
	 */
	public function __construct($file = '')
	{
	  parent::__construct('1.0', 'utf-8');

		//$this->validateOnParse = true;
		$this->preserveWhiteSpace = false;
		$this->formatOutput = true;

		$this->file  = $file;
		$this->xpath = null;
	}

	/**
	  @brief Open a XML document.
	  @param $file Name of the document file.
	  @return XML object on success, or null on failure (file not exists etc.).
	 */
	public static function open($file)
	{
		if (!is_file($file))
			return null;

		$class = __CLASS__;
		$doc = new $class($file);

  	$doc->load($file);
	  $doc->xpath = new DOMXPath($doc);
		self::setXMLNamespace($doc);
    
		return $doc;
	}

	/**
	  @brief Open a XML document from a string.
	  @param $input The document as a string.
	  @return XML object on success.
	 */
	public static function openFromString($input)
	{
		$class = __CLASS__;
		$doc = new $class();
		
		/* TODO: add document type definition */
		
		$doc->loadXML($input);
		$doc->xpath = new DOMXPath($doc);
		self::setXMLNamespace($doc);
		
		//$doc->validate();
		
		return $doc;
	}
	/**
	  @brief Defines XML namespaces that will be used for IQ protocol
	  @param $xpath Document DOM XPath object passed as reference 
	 */	
	public static function setXmlNamespace(&$doc)
	{
		$doc->xpath->registerNamespace("auth","webmail:iq:auth");  
		$doc->xpath->registerNamespace("accounts","webmail:iq:accounts");  
		$doc->xpath->registerNamespace("folders","webmail:iq:folders");  
		$doc->xpath->registerNamespace("items","webmail:iq:items");
		$doc->xpath->registerNamespace("private","webmail:iq:private"); 
		$doc->xpath->registerNamespace("domain","webmail:iq:domain"); 
		$doc->xpath->registerNamespace("public","webmail:iq:public");  
		$doc->xpath->registerNamespace("message","webmail:iq:message"); 
		$doc->xpath->registerNamespace("freebusy","webmail:iq:freebusy");  
		$doc->xpath->registerNamespace("spellchecker","webmail:iq:spellchecker"); 
		$doc->xpath->registerNamespace("import","webmail:iq:import");  

	}
	/**
	  @brief Save the document.
	  @param $file Optional new name of the document file.
	 */
	public function save($file = '')
	{
		if (($file === '')
			&& ($this->file === ''))
		{
			throw new Exc('Invalid file name');
		}

		parent::save(($file !== '') ? $file : $this->file);
	}

	/**
	  @brief Invoke a XPath query over the document.
	  @param $xpath The XPath query.
	  @param $node Optional context node for doing relative XPath queries.
	  @returns DOMNodeList of all nodes matching the XPath query.
	 */
	public function query($xpath, $node = null)
	{
		return $this->xpath
			? $this->xpath->query($xpath, $node ? $node : $this)
			: array();
	}

	/**
	  @brief Get the first node of the node specified by a XPath query.
	  @param $xpath The XPath query.
	  @param $node Optional context node for doing relative XPath queries.
	  @return The node on success, or null on failure.
	 */
	public function getNode($xpath, $node = null)
	{
		$nodes = $this->query($xpath, $node);
		if ($nodes->length > 1)
			throw new Exc($xpath . ': More nodes found');
		if ($nodes->length == 0)
			return null;

		return $nodes->item(0);
	}

	/**
	  @brief Get value of the node specified by a XPath query.
	  @param $xpath The XPath query.
	  @param $node Optional context node for doing relative XPath queries.
	  @return Value of the node on success, or '' on failure.
	 */
	public function getNodeValue($xpath, $node = null)
	{
		if (!$node = $this->getNode($xpath, $node))
			 return false;
		
		return $node->nodeValue;
	}

	/**
	  @brief Get value of an attribute of the node specified by a XPath query.
	  @param $xpath The XPath query.
	  @param $name Name of the attribute.
	  @param $node Optional context node for doing relative XPath queries.
	  @return Value of the attribute on success, or '' on failure.
	 */
	public function getNodeAttribute($xpath, $name, $node = null)
	{
		$node = $this->getNode($xpath, $node);
		if ($node === null)
			throw new Exc('Node not found');

		return $node->getAttribute($name);
	}


	/**
	  @brief Convert a string to be safe in XML document.
	  @param $str The string.
	  @return The converted string.
	 */
	public static function encode($str)
	{
		return htmlspecialchars($str, ENT_COMPAT, 'UTF-8');
	}
	
	/**
	  @brief Returns an string value of node including all subnodes
	  @param $node DOM Node object
	  @return The output string.
	  @author Pavel Perna
	 */	
	public function getNodeContent($node)
	{	 
		  //Null all local variables
		  $sStart = $sClose = $sReturn = $sSubReturn = '';
		  $bHasNodes = $bHasValue = false;
		  
		  /* Get all element's attributes */
		  $sAttributes = '';
		  if (@$node->hasAttributes()){
				foreach ($node->attributes as $attribute)
					if ($attribute->value!="") $sAttributes.=' '.$attribute->name.'="'.$attribute->value.'"';
		  }
	  
		  /* Get xmlns */

		  if (@$node->namespaceURI && $node->nodeName=='query')
		  {
          $sAttributes.=' xmlns="'.$node->namespaceURI.'"';
		  }   

		  /* Recurently process all subnodes if there are any */
		  if (@$node->hasChildNodes()) 
		  {
				foreach ($node->childNodes as $childNode){
					  $sSubReturn.=self::getNodeContent($childNode);
				}
				
				$sReturn = $sSubReturn; 
				$bHasNodes = true;
				
		  /* If there are no child nodes , end tree leaf and return CDATA */ 
		  } else {	  
				$sReturn=$node->nodeValue;
				if ($node->nodeValue) $bHasValue = true;
		  }
		  
		  /* Compose start and close tag */
		  if (@$node->tagName){
			$bPair = ($bHasValue || $bHasNodes);
			$sStart = '<'.$node->tagName.$sAttributes.($bPair?">":"/>");
			$sClose = ($bPair?'</'.$node->tagName.'>':'');
		  }
	
		  return $sStart.$sReturn.$sClose;
	}
}

?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/server/inc/xml.php <==
<?php

/**
  @class XML
  @brief Class for editing and querying XML documents.
  @author Josef Hala
  @author Martin Zlomek
  @date 21-02-2006
 */
class XML extends DOMDocument {
	/** Name of the document file. */
	private $file;

	/** XPath object of the document. */
	private $xpath;

	/**
	  @brief Create a new XML document object.
	  @param $file Optional name of the document file.
	 */
	public function __construct($file = '')
	{
	  parent::__construct('1.0', 'utf-8');

		//$this->validateOnParse = true;
		$this->preserveWhiteSpace = false;
		$this->formatOutput = true;

		$this->file  = $file;
		$this->xpath = null;
	}

	/**
	  @brief Open a XML document.
	  @param $file Name of the document file.
	  @return XML object on success, or null on failure (file not exists etc.).
	 */
	public static function open($file)
	{
		if (!is_file($file))
			return null;

		$class = __CLASS__;
		$doc = new $class($file);

  	$doc->load($file);
	  $doc->xpath = new DOMXPath($doc);
		self::setXMLNamespace($doc);
    
		return $doc;
	}

	/**
	  @brief Open a XML document from a string.
	  @param $input The document as a string.
	  @return XML object on success.
	 */
	public static function openFromString($input)
	{
		$class = __CLASS__;
		$doc = new $class();
			
		$doc->loadXML($input);
		$doc->xpath = new DOMXPath($doc);
		self::setXMLNamespace($doc);
		
		//$doc->validate();
		
		return $doc;
	}
	/**
	  @brief Defines XML namespaces that will be used for IQ protocol
	  @param $xpath Document DOM XPath object passed as reference 
	 */	
	public static function setXmlNamespace(&$doc)
	{
		$doc->xpath->registerNamespace("auth","webmail:iq:auth");  
		$doc->xpath->registerNamespace("accounts","webmail:iq:accounts");  
		$doc->xpath->registerNamespace("folders","webmail:iq:folders");  
		$doc->xpath->registerNamespace("items","webmail:iq:items");
		$doc->xpath->registerNamespace("private","webmail:iq:private"); 
		$doc->xpath->registerNamespace("domain","webmail:iq:domain"); 
		$doc->xpath->registerNamespace("public","webmail:iq:public");  
		$doc->xpath->registerNamespace("message","webmail:iq:message"); 
		$doc->xpath->registerNamespace("freebusy","webmail:iq:freebusy");  
		$doc->xpath->registerNamespace("spellchecker","webmail:iq:spellchecker"); 
		$doc->xpath->registerNamespace("import","webmail:iq:import");  

	}
	/**
	  @brief Save the document.
	  @param $file Optional new name of the document file.
	 */
	public function save($file = '')
	{
		if (($file === '')
			&& ($this->file === ''))
		{
			throw new Exc('xml_invalid_file_name');
		}
		@parent::save(($file !== '') ? $file : $this->file);
	}

	/**
	  @brief Invoke a XPath query over the document.
	  @param $xpath The XPath query.
	  @param $node Optional context node for doing relative XPath queries.
	  @returns DOMNodeList of all nodes matching the XPath query.
	 */
	public function query($xpath, $node = null)
	{
		return $this->xpath
			? $this->xpath->query($xpath, $node ? $node : $this)
			: array();
	}

	/**
	  @brief Get the first node of the node specified by a XPath query.
	  @param $xpath The XPath query.
	  @param $node Optional context node for doing relative XPath queries.
	  @return The node on success, or null on failure.
	 */
	public function getNode($xpath, $node = null)
	{
		$nodes = $this->query($xpath, $node);
		if ($nodes->length > 1)
			throw new Exc('xml_xpath_more_nodes_found',$xpath);
		if ($nodes->length == 0)
			return null;

		return $nodes->item(0);
	}

	/**
	  @brief Get value of the node specified by a XPath query.
	  @param $xpath The XPath query.
	  @param $node Optional context node for doing relative XPath queries.
	  @return Value of the node on success, or '' on failure.
	 */
	public function getNodeValue($xpath, $node = null)
	{
		if (!$node = $this->getNode($xpath, $node))
			 return false;
		
		return $node->nodeValue;
	}

	/**
	  @brief Get value of an attribute of the node specified by a XPath query.
	  @param $xpath The XPath query.
	  @param $name Name of the attribute.
	  @param $node Optional context node for doing relative XPath queries.
	  @return Value of the attribute on success, or '' on failure.
	 */
	public function getNodeAttribute($xpath, $name, $node = null)
	{
		$node = $this->getNode($xpath, $node);
		if ($node === null)
			throw new Exc('xml_node_does_not_exist',$name);

		return $node->getAttribute($name);
	}


	/**
	  @brief Convert a string to be safe in XML document.
	  @param $str The string.
	  @return The converted string.
	 */
	public static function encode($str)
	{
		return htmlspecialchars($str, ENT_COMPAT, 'UTF-8');
	}
	
	/**
	  @brief Returns an string value of node including all subnodes
	  @param $node DOM Node object
	  @return The output string.
	  @author Pavel Perna
	 */	
	public function getNodeContent($node)
	{	 
		  //Null all local variables
		  $sStart = $sClose = $sReturn = $sSubReturn = '';
		  $bHasNodes = $bHasValue = false;
		  
		  /* Get all element's attributes */
		  $sAttributes = '';
		  if (@$node->hasAttributes()){
				foreach ($node->attributes as $attribute)
					if ($attribute->value!="") $sAttributes.=' '.$attribute->name.'="'.$attribute->value.'"';
		  }
	  
		  /* Get xmlns */

		  if (@$node->namespaceURI && $node->nodeName=='query')
		  {
          $sAttributes.=' xmlns="'.$node->namespaceURI.'"';
		  }   

		  /* Recurently process all subnodes if there are any */
		  if (@$node->hasChildNodes()) 
		  {
				foreach ($node->childNodes as $childNode){
					  $sSubReturn.=self::getNodeContent($childNode);
				}
				
				$sReturn = $sSubReturn; 
				$bHasNodes = true;
				
		  /* If there are no child nodes , end tree leaf and return CDATA */ 
		  } else {	  
				$sReturn=$node->nodeValue;
				if ($node->nodeValue) $bHasValue = true;
		  }
		  
		  /* Compose start and close tag */
		  if (@$node->tagName){
			$bPair = ($bHasValue || $bHasNodes);
			$sStart = '<'.$node->tagName.$sAttributes.($bPair?">":"/>");
			$sClose = ($bPair?'</'.$node->tagName.'>':'');
		  }
	
		  return $sStart.$sReturn.$sClose;
	}
}

?>
